# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv, fields

class partner_accounts_installer(osv.osv_memory):

    _name = "partner.accounts.installer"
    _inherit = 'res.config.installer'

    def _default_company(self, cr, uid, context=None):
        """set to default company of user"""
        if context is None: context = {}
        
        user = self.pool.get('res.users').browse(cr, uid, uid, context=context)
        return user.company_id and user.company_id.id or False

    _columns = {
        'company_id': fields.many2one('res.company', 'Company',required=True),
        'customer_parent_account': fields.many2one('account.account', 'Default receivable customer account', help="Default receivable account for customers (4300000 for example)",required=True),
        'supplier_parent_account': fields.many2one('account.account', 'Default payable supplier account', help="Default payable account for suppliers (4100000 for example)",required=True)
    }

    _defaults = {
        'company_id': _default_company
    }

    def execute(self, cr, uid, ids, context=None):
        if context is None:
            context = {}

        sequence_facade = self.pool.get('ir.sequence')
        super(partner_accounts_installer, self).execute(cr, uid, ids, context=context)

        record = self.browse(cr, uid, ids, context=context)[0]
        company_id = record.company_id.id

        #Miramos si existe ya una secuencia adecuada para los clientes de esta compañía...
        company_customer_seq_id = sequence_facade.search(cr, uid, [('code', '=', 'res.partner.customer'),('company_id', '=', company_id)])
        if not company_customer_seq_id:
            seq_name="%s Customer Default Account"%(record.company_id.name)
            sequence_facade.create(cr,uid,{'name':seq_name,'code':'res.partner.customer','active':True,'company_id':company_id,'padding':5,'number_next':1})

        #Miramos si existe ya una secuencia adecuada para los proveedores de esta compañía...
        company_supplier_seq_id = sequence_facade.search(cr, uid, [('code', '=', 'res.partner.supplier'),('company_id', '=', company_id)])
        if not company_supplier_seq_id:
            seq_name="%s Supplier Default Account"%(record.company_id.name)
            sequence_facade.create(cr,uid,{'name':seq_name,'code':'res.partner.supplier','active':True,'company_id':company_id,'padding':5,'number_next':1})

        self.pool.get('res.company').write(cr, uid, [company_id], {'customer_parent_account': record.customer_parent_account.id,
                                                                    'supplier_parent_account': record.supplier_parent_account.id})

partner_accounts_installer()