# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

"""Creates personal accounts for partner in Openerp"""

from osv import osv
from tools.translate import _

class res_partner(osv.osv):
    """Creates personal accounts for partner in Openerp"""

    _inherit = 'res.partner'

    def _get_code_digits(self, cr, uid, context=None, company_id=None):
        """
        Returns the default code size for the accounts.

        To figure out the number of digits of the accounts it look at the
        code size of the default receivable account of the company
        (or user's company if any company is given).
        """
        if context is None:
            context = {}

        if not company_id:
            company_id = self.pool.get('res.users').browse(cr, uid, uid).company_id
            if context.get('company_id'):
                company_id = context['company_id']

            account_id =  company_id.customer_parent_account
        elif isinstance(company_id, (int, long)):
            company_id = self.pool.get('res.users').browse(cr, uid, uid).company_id
                
        return len(account_id.code) or 6

    def _update_cascade_account_chart(self, cr, uid, sequence, companies, account_type, partner_obj, context=None):
        """
            Performs a cascade update in all of a given company's children in order to share some customers or suppliers between them
        """
        if context is None:
            context = {}
        root_uid = 1
        property_facade = self.pool.get('ir.property')
        for company in companies:
            if sequence:
                context.update({'company_id':company})
                if account_type=='receivable':
                    parent_acc_id = company.customer_parent_account
                    prop_name = 'property_account_receivable'
                    field_description = 'Account Receivable'
                else:
                    parent_acc_id = company.supplier_parent_account
                    prop_name = 'property_account_payable'
                    field_description = 'Account Payable'

                account_code = parent_acc_id.code[:2] + sequence.zfill(self._get_code_digits(cr, uid, context=context) - 2)
                exit = False
                while not exit:
                    existing_account = self.pool.get('account.account').search(cr, root_uid, [('code','=',account_code), ('company_id','=',company.id)])
                    if existing_account:
                        #Para esa compañía, ya está asignado ese código de cuenta, por tanto, buscamos el siguiente
                        account_code = str(int(account_code)+1)
                    else:
                        exit = True

                account_vals = {
                    'name': partner_obj.name,
                    'code': account_code,
                    'type': account_type,
                    'user_type': parent_acc_id.user_type.id,
                    'parent_id': parent_acc_id.parent_id.id,
                    'reconcile': True,
                    'company_id': company.id
                }

                account_id = self.pool.get('account.account').create(cr, root_uid, account_vals)

                #Gestiono las properties, ya que serán contra las que trabajen las facturas...
                #Busco si existe para esa compañía y empresa una property previa...
                existing_prop = property_facade.search(cr, root_uid, [('company_id','=',company.id), ('res_id','=','res.partner,'+str(partner_obj.id)+''), ('name','=', prop_name)])
                if existing_prop:
                    vals_prop = {
                        'value_reference': 'account.account,%s' %(account_id),
                    }
                    property_facade.write(cr, root_uid, existing_prop[0], vals_prop)
                else:
                    vals_prop = {
                        'name': "%s" %(prop_name),
                        'company_id': company.id,
                        'fields_id': self.pool.get('ir.model.fields').search(cr, root_uid,[('field_description','=', field_description)])[0],
                        'type': 'many2one',
                        'value_reference': 'account.account,%s' %(account_id),
                        'res_id': 'res.partner,%s' %(partner_obj.id)
                        }
                    property_facade.create(cr, root_uid, vals_prop)

            else:
                raise osv.except_osv(_("Error"), _("Have you already run 'Accounting-->Configuration-->Financial Accounting-->Configure default customer/supplier accounts' wizard for this company?"))
        return True


    def create(self, cr, uid, vals, context=None):
        """Sequence only assigned to customer or supplier partners"""
        if context is None: context = {}
        root_uid = 1
        company_facade = self.pool.get('res.company')
        partner_id = super(res_partner, self).create(cr, root_uid, vals, context = context)
        partner_obj = self.browse(cr, uid, partner_id)
        #current company
        company_id = self.pool.get('res.users').browse(cr, uid, uid).company_id
        if context.get('company_id'):
            company_id = context['company_id']

        if company_id.customer_parent_account: # searches for parent account id
            parent_acc_id = company_id.customer_parent_account
        else:
            raise osv.except_osv(_("Error"), _("Should configure default receivable customer account in company to continue."))

        #receivable acount
        if ((vals.get('property_account_receivable') and vals['property_account_receivable'] == parent_acc_id.id) or not vals.get('property_account_receivable')) and vals.get('customer'):
            cust_sequence_ids = self.pool.get('ir.sequence').search(cr, uid, [('code', '=', 'res.partner.customer')])
            if not cust_sequence_ids:
                raise osv.except_osv(_("Error"), _("Have you installed 'partner_accounts' module?"))
            else:
                cust_sequence = self.pool.get('ir.sequence').get(cr, uid, 'res.partner.customer')

            if cust_sequence:
                #Vamos a controlar que el cliente/proveedor que estamos creando no tenga compañía o su compañía tenga hijas para actualizar los planes contables en cascada
                if not partner_obj.company_id or partner_obj.company_id.child_ids:
                    if not partner_obj.company_id:
                        #Cogemos la compañia padre...
                        parent_company = company_facade.browse(cr, root_uid, company_facade.search(cr, root_uid, [('parent_id','=',False)]))
                        parent_company = parent_company and parent_company[0] or False
                        company_children = parent_company.child_ids
                    else:
                        company_children = partner_obj.company_id.child_ids
                    self._update_cascade_account_chart(cr, root_uid, cust_sequence, company_children, 'receivable', partner_obj, context=context)
                else:
                    self._update_cascade_account_chart(cr, root_uid, cust_sequence, [partner_obj.company_id], 'receivable', partner_obj, context=context)
            else:
                raise osv.except_osv(_("Error"), _("Have you already run 'Accounting-->Configuration-->Financial Accounting-->Configure default customer/supplier accounts' wizard for this company?"))

        #payable account
        if company_id.supplier_parent_account: # searches for parent account id
            parent_acc_id =  company_id.supplier_parent_account
        else:
            raise osv.except_osv(_("Error"), _("Should configure default payable supplier account in company to continue."))

        if ((vals.get('property_account_payable') and vals['property_account_payable'] == parent_acc_id.id) or not vals.get('property_account_payable')) and vals.get('supplier'):
            sup_sequence_ids = self.pool.get('ir.sequence').search(cr, uid, [('code', '=', 'res.partner.supplier')])
            if not sup_sequence_ids:
                raise osv.except_osv(_("Error"), _("Have you installed 'partner_accounts' module?"))
            else:
                sup_sequence = self.pool.get('ir.sequence').get(cr, uid, 'res.partner.supplier')
            if sup_sequence:
                #Vamos a controlar que el cliente/proveedor que estamos creando no tenga compañía o su compañía tenga hijas para actualizar los planes contables en cascada
                if not partner_obj.company_id or partner_obj.company_id.child_ids:
                    if not partner_obj.company_id:
                        #Cogemos la compañia padre...
                        parent_company = company_facade.browse(cr, root_uid, company_facade.search(cr, root_uid, [('parent_id','=',False)]))
                        parent_company = parent_company and parent_company[0] or False
                        company_children = parent_company.child_ids
                    else:
                        company_children = partner_obj.company_id.child_ids
                    self._update_cascade_account_chart(cr, root_uid, sup_sequence, company_children, 'payable', partner_obj, context=context)
                else:
                    self._update_cascade_account_chart(cr, root_uid, sup_sequence, [partner_obj.company_id], 'payable', partner_obj, context=context)

            else:
                raise osv.except_osv(_("Error"), _("Have you already run 'Accounting-->Configuration-->Financial Accounting-->Configure default customer/supplier accounts' wizard for this company?"))

        return partner_id

res_partner()
